﻿using System;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Ink;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;

namespace CustomComboEditorColumn
{
    /// <summary>
    /// Provides utility methods that can be used to get child or parent objects of another object by examining the visual tree.
    /// </summary>
    /// <remarks>
    /// Author: Borchardt
    /// Date: 2010-10-20
    /// </remarks>
    public class RelationshipHelper
    {
        #region Get child

        /// <summary>
        /// Gets the child object of a specific object by examining the visual tree.
        /// </summary>
        /// <param name="parent">The object to which the child is wanted.</param>
        /// <param name="name">The name of the child which is wanted.</param>
        /// <returns>Returns the child dependency object.</returns>
        public static FrameworkElement GetChildByName(FrameworkElement parent, String name)
        {
            if (parent == null)
            {
                return null;
            }
            if (parent.Name == name)
            {
                return parent;
            }

            FrameworkElement child = null;
            for (int i = 0; i < VisualTreeHelper.GetChildrenCount(parent); i++)
            {
                FrameworkElement temp = VisualTreeHelper.GetChild(parent, i) as FrameworkElement;
                if (temp == null) { continue; }
                child = GetChildByName(temp, name);
                if (child != null) { break; }
            }
            return child;
        }


        /// <summary>
        /// Gets the child object of a specific object by examining the visual tree.
        /// </summary>
        /// <param name="parent">The object to which the child is wanted.</param>
        /// <param name="type">The type of the child which is wanted.</param>
        /// <returns>Returns the child dependency object.</returns>
        public static DependencyObject GetChildByType(DependencyObject parent, Type type)
        {
            if (parent == null)
            {
                return null;
            }
            if (parent.GetType() == type)
            {
                return parent;
            }

            DependencyObject child = null;
            for (int i = 0; i < VisualTreeHelper.GetChildrenCount(parent); i++)
            {
                DependencyObject temp = VisualTreeHelper.GetChild(parent, i);
                child = GetChildByType(temp, type);
                if (child != null)
                {
                    break;
                }
            }
            return child;
        }

        #endregion

        #region Get parent

        /// <summary>
        /// Gets the parent object by its type of a specific object by examining the visual tree.
        /// </summary>
        /// <param name="child">The object to which the parent is wanted.</param>
        /// <param name="parentType">The type of the parent which is wanted.</param>
        /// <returns>Returns the parent dependency object.</returns>
        public static DependencyObject GetParentByType(DependencyObject child, Type parentType)
        {
            DependencyObject tempParent = VisualTreeHelper.GetParent(child);
            if (tempParent != null)
            {
                Type tempParentType = tempParent.GetType();
                if (tempParentType == parentType)
                {
                    return tempParent;
                }
                else
                {
                    return GetParentByType(tempParent, parentType);
                }
            }

            return null;
        }

        /// <summary>
        /// Gets the parent object by its base type of a specific object by examining the visual tree.
        /// </summary>
        /// <param name="child">The object to which the parent is wanted.</param>
        /// <param name="parentType">The type of the parent which is wanted.</param>
        /// <returns>Returns the parent dependency object.</returns>
        public static DependencyObject GetParentByBaseType(DependencyObject child, Type parentType)
        {
            DependencyObject tempParent = VisualTreeHelper.GetParent(child);
            if (tempParent != null)
            {
                Type tempParentType = tempParent.GetType();
                while (tempParentType != parentType && tempParentType != null)
                {
                    tempParentType = tempParentType.BaseType;
                }
                if (tempParentType == parentType)
                {
                    return tempParent;
                }
                else
                {
                    return GetParentByBaseType(tempParent, parentType);
                }
            }

            return null;
        }

        #endregion
    }
}
